hasGlobalContext property

bool get hasGlobalContext

Check if global context is available Global context is set via ReactiveNotifier.initContext() and persists throughout the app lifecycle regardless of builder state

Implementation

bool get hasGlobalContext => ViewModelContextNotifier.hasGlobalContext();